require 'cocoapods'
require 'fileutils'
require 'json'
require 'xcodeproj'
require 'cocoapods-downloader'

module Pod
  class Command
    class Devops < Command
      self.summary = <<-SUMMARY
        Install components through devops/components.json
      SUMMARY
      self.description = <<-DESC
        Install components through devops/components.json
      DESC
      self.arguments = []

      def run
        # do nothing
        Pod::UI.puts '插件生效'
      end
    end
  end
end

module CocoapodsSoulComponentPlugin
  $components = []

  class Soul_Component
    attr_accessor :name, :local, :submodule, :version, :git, :branch, :path, :commit

    def initialize(name, local, submodule, version, git, branch, path)
      @name = name
      @local = local
      @submodule = submodule
      @version = version
      @git = git
      @branch = branch
      @path = path
      @commit = commit
    end
  end

  def self.use_components
    File.exist?(component_file_path)
  end

  def self.root_path
    Pod::Config.instance.installation_root.to_s
  end

  def self.podfile_path
    root_path + '/Podfile'
  end

  def self.component_file_path
    root_path + '/devops/component.json'
  end

  Pod::HooksManager.register('cocoapods-soul-component-plugin', :pre_install) do |_context|
    Pod::UI.puts '插件 pre_install'
  end
  Pod::HooksManager.register('cocoapods-soul-component-plugin', :post_install) do |_context|
    Pod::UI.puts '插件 post_install'
  end
  Pod::HooksManager.register('cocoapods-soul-component-plugin', :source_provider) do |_context, _|
    Pod::UI.puts '插件 source_provider'
  end

  def self.post_run
    return if use_components == false

    Pod::UI.puts '添加component.json文件引用'
    project = Xcodeproj::Project.open(Pod::Config.instance.sandbox.project_path)
    # 获取主group
    group = project.main_group
    group.set_source_tree('SOURCE_ROOT')
    # 向group中添加 文件引用
    file_ref = group.new_reference(Pod::Config.instance.sandbox.root + '../devops/component.json')
    # podfile_local排序
    podfile_local_group = group.children.last
    group.children.pop
    group.children.unshift(podfile_local_group)
    project.save
  end

  def self.pre_run
    time = Time.now.to_i

    json = File.read(self.component_file_path)
    obj = JSON.parse(json)
    dependencies = obj['dependencies']

    dependencies.each do |each|
      $components.push(Soul_Component.new(each[0], each[1]['local'], each[1]['submodule'], each[1]['version'], each[1]['git'], each[1]['branch'], each[1]['path']))
    end

    $components.each do |each|
      if each.path == nil
        if each.submodule == false
          if each.local == true
            local_path = each.name + each.branch.to_s + each.commit.to_s
            target_path = root_path + '/LocalPods/' + local_path
            each.path = target_path

            if File.exist?(target_path) == false
              options = { git: each.git, commit: each.commit, branch: each.branch }
              options = Pod::Downloader.preprocess_options(options)
              downloader = Pod::Downloader.for_target(target_path, options)
              downloader.download
              downloader.checkout_options #=> { :git => 'example.com', :commit => 'd7f410490dabf7a6bde665ba22da102c3acf1bd9' }
            end
          end
        else
          if each.local == true
            each.path = "SOPods/#{each.name}"
          end
        end
      end
    end
  end
end

# 对install进行hook
module Pod
  class Command
    class Install < Command
      alias old_run run
      def run
        CocoapodsSoulComponentPlugin.pre_run if CocoapodsSoulComponentPlugin.use_components
        old_run
        CocoapodsSoulComponentPlugin.post_run if CocoapodsSoulComponentPlugin.use_components
      end
    end
  end

  class Dependency
    alias old_initialize initialize

    def initialize(name = nil, *requirements)
      if requirements == nil
        old_initialize(name)
        return
      end

      # if name == "SLog"
      #   puts name, requirements
      # end

      $components.each do |each|
        if each.name == name
          if each.local == true
            requirements = [{ :name => each.name, :path => each.path }]
          else
            requirements = [each.version]
          end
        end
      end

      old_initialize(name, *requirements)
    end
  end
end
